@@ -8,7 +8,7 @@ DEFAULT_META_ANCHOR = {"name": None,
                        "type": ["blackbox"],
                        "explanations": ["local"],
                        "params": {},
-                       "version": None}
+                       "version": None}  # type: dict
 """
 Default anchor metadata.
 """
@@ -35,7 +35,7 @@ DEFAULT_META_CEM = {"name": None,
                     "type": ["blackbox", "tensorflow", "keras"],
                     "explanations": ["local"],
                     "params": {},
-                    "version": None}
+                    "version": None}  # type: dict
 """
 Default CEM metadata.
 """
@@ -57,7 +57,7 @@ DEFAULT_META_CF = {"name": None,
                    "type": ["blackbox", "tensorflow", "keras"],
                    "explanations": ["local"],
                    "params": {},
-                   "version": None}
+                   "version": None}  # type: dict
 """
 Default counterfactual metadata.
 """
@@ -76,7 +76,7 @@ DEFAULT_META_CFP = {"name": None,
                     "type": ["blackbox", "tensorflow", "keras"],
                     "explanations": ["local"],
                     "params": {},
-                    "version": None}
+                    "version": None}  # type: dict
 """
 Default counterfactual prototype metadata.
 """
@@ -104,7 +104,8 @@ KERNEL_SHAP_PARAMS = [
     'kwargs',
 ]
 """
-KernelShap parameters updated and return in metadata['params'].
+KernelShap parameters updated and returned in ``metadata['params']``.
+See :py:class:`alibi.explainers.shap_wrappers.KernelShap`.
 """
 
 DEFAULT_META_KERNEL_SHAP = {
@@ -172,7 +173,8 @@ TREE_SHAP_PARAMS = [
     'kwargs'
 ]
 """
-TreeShap parameters updated and return in metadata['params'].
+TreeShap parameters updated and returned in ``metadata['params']``.
+See :py:class:`alibi.explainers.shap_wrappers.TreeShap`.
 """
 
 DEFAULT_META_TREE_SHAP = {
