@@ -1,16 +1,21 @@
 import logging
+from typing import Any, Optional, Tuple
+
 import numpy as np
-from sklearn.neighbors import KDTree
-from sklearn.neighbors import KNeighborsClassifier
-from typing import Tuple, Any
+from sklearn.neighbors import KDTree, KNeighborsClassifier
 
 logger = logging.getLogger(__name__)
 
 
-class TrustScore(object):
+class TrustScore:
 
-    def __init__(self, k_filter: int = 10, alpha: float = 0., filter_type: str = None,
-                 leaf_size: int = 40, metric: str = 'euclidean', dist_filter_type: str = 'point') -> None:
+    def __init__(self,
+                 k_filter: int = 10,
+                 alpha: float = 0.,
+                 filter_type: Optional[str] = None,
+                 leaf_size: int = 40,
+                 metric: str = 'euclidean',
+                 dist_filter_type: str = 'point') -> None:
         """
         Initialize trust scores.
 
@@ -21,15 +26,15 @@ class TrustScore(object):
         alpha
             Fraction of instances to filter out to reduce impact of outliers.
         filter_type
-            Filter method; either 'distance_knn' or 'probability_knn'
+            Filter method: ``'distance_knn'`` | ``'probability_knn'``.
         leaf_size
             Number of points at which to switch to brute-force. Affects speed and memory required to build trees.
-            Memory to store the tree scales with n_samples / leaf_size.
+            Memory to store the tree scales with `n_samples / leaf_size`.
         metric
-            Distance metric used for the tree. See sklearn's DistanceMetric class for a list of available metrics.
+            Distance metric used for the tree. See `sklearn` DistanceMetric class for a list of available metrics.
         dist_filter_type
-            Use either the distance to the k-nearest point (dist_filter_type = 'point') or
-            the average distance from the first to the k-nearest point in the data (dist_filter_type = 'mean').
+            Use either the distance to the k-nearest point (``dist_filter_type = 'point'``) or
+            the average distance from the first to the k-nearest point in the data (``dist_filter_type = 'mean'``).
         """
         self.k_filter = k_filter
         self.alpha = alpha
@@ -47,7 +52,7 @@ class TrustScore(object):
         Parameters
         ----------
         X
-            Data
+            Data.
 
         Returns
         -------
@@ -70,9 +75,9 @@ class TrustScore(object):
         Parameters
         ----------
         X
-            Data
+            Data.
         Y
-            Predicted class labels
+            Predicted class labels.
 
         Returns
         -------
@@ -93,18 +98,18 @@ class TrustScore(object):
         X_keep, Y_keep = X[keep_id, :], Y[keep_id]
         return X_keep, Y_keep
 
-    def fit(self, X: np.ndarray, Y: np.ndarray, classes: int = None) -> None:
+    def fit(self, X: np.ndarray, Y: np.ndarray, classes: Optional[int] = None) -> None:
         """
         Build KDTrees for each prediction class.
 
         Parameters
         ----------
         X
-            Data
+            Data.
         Y
             Target labels, either one-hot encoded or the actual class label.
         classes
-            Number of prediction classes, needs to be provided if Y equals the predicted class.
+            Number of prediction classes, needs to be provided if `Y` equals the predicted class.
         """
         self.classes = classes if classes is not None else Y.shape[1]
         self.kdtrees = [None] * self.classes  # type: Any
@@ -156,8 +161,8 @@ class TrustScore(object):
         k
             Number of nearest neighbors used for distance calculation.
         dist_type
-            Use either the distance to the k-nearest point (dist_type = 'point') or
-            the average distance from the first to the k-nearest point in the data (dist_type = 'mean').
+            Use either the distance to the k-nearest point (``dist_type = 'point'``) or
+            the average distance from the first to the k-nearest point in the data (``dist_type = 'mean'``).
 
         Returns
         -------
@@ -173,7 +178,8 @@ class TrustScore(object):
                            'be queried.'.format(X.shape, X.reshape(X.shape[0], -1).shape))
             X = X.reshape(X.shape[0], -1)
 
-        d = np.tile(None, (X.shape[0], self.classes))  # init distance matrix: [nb instances, nb classes]
+        # init distance matrix: [nb instances, nb classes]
+        d = np.tile(None, (X.shape[0], self.classes))  # type: np.ndarray
 
         for c in range(self.classes):
             d_tmp = self.kdtrees[c].query(X, k=k)[0]  # get k nearest neighbors for each class
