@@ -1,17 +1,18 @@
-import PIL
+import logging
+import pkgutil
+import tarfile
 from io import BytesIO, StringIO
+from typing import Optional, Tuple, Union
+
 import numpy as np
 import pandas as pd
-import pkgutil
+import PIL
 import requests
+import tensorflow.keras as keras
 from requests import RequestException
 from sklearn.preprocessing import LabelEncoder
-import tarfile
-from typing import Tuple, Union
-import logging
-from alibi.utils.data import Bunch
 
-import tensorflow.keras as keras
+from alibi.utils.data import Bunch
 
 logger = logging.getLogger(__name__)
 
@@ -38,9 +39,9 @@ def load_cats(target_size: tuple = (299, 299), return_X_y: bool = False) -> Unio
     Parameters
     ----------
     target_size
-        Size of the returned images, used to crop images for a specified model input size
+        Size of the returned images, used to crop images for a specified model input size.
     return_X_y
-        If true, return features X and labels y as numpy arrays, if False return a Bunch object
+        If ``True``, return features `X` and labels `y` as `numpy` arrays. If ``False`` return a `Bunch` object
 
     Returns
     -------
@@ -48,17 +49,18 @@ def load_cats(target_size: tuple = (299, 299), return_X_y: bool = False) -> Unio
         Bunch object with fields 'data', 'target' and 'target_names'. Both `targets` and `target_names` are taken from
         the original Imagenet.
     (data, target)
-        Tuple if ``return_X_y`` is true
+        Tuple if ``return_X_y=True``.
     """
-    tar = tarfile.open(fileobj=BytesIO(pkgutil.get_data(__name__, "data/cats.tar.gz")), mode='r:gz')
+    tar = tarfile.open(fileobj=BytesIO(pkgutil.get_data(__name__, "data/cats.tar.gz")),  # type: ignore[arg-type]
+                       mode='r:gz')
     images = []
     target = []
     target_names = []
     for member in tar.getmembers():
         # data
-        img = tar.extractfile(member).read()
+        img = tar.extractfile(member).read()  # type: ignore[union-attr]
         img = PIL.Image.open(BytesIO(img))
-        img = np.expand_dims(img.resize(target_size), axis=0)  # type: ignore
+        img = np.expand_dims(img.resize(target_size), axis=0)
         images.append(img)
 
         # labels
@@ -67,10 +69,10 @@ def load_cats(target_size: tuple = (299, 299), return_X_y: bool = False) -> Unio
         target_names.append('_'.join(name).split('.')[0])
     tar.close()
 
-    images = np.concatenate(images, axis=0)
+    images = np.concatenate(images, axis=0)  # type: ignore[assignment]
     targets = np.asarray(target)
     if return_X_y:
-        return images, targets  # type: ignore
+        return images, targets  # type: ignore[return-value] # TODO: allow redefiniton
     else:
         return Bunch(data=images, target=targets, target_names=target_names)
 
@@ -90,7 +92,7 @@ def fetch_movie_sentiment(return_X_y: bool = False, url_id: int = 0) -> Union[Bu
     Parameters
     ----------
     return_X_y
-        If true, return features X and labels y as Python lists, if False return a Bunch object
+        If ``True``, return features `X` and labels `y` as `Python` lists. If ``False`` return a `Bunch` object.
     url_id
         Index specifying which URL to use for downloading
 
@@ -99,7 +101,7 @@ def fetch_movie_sentiment(return_X_y: bool = False, url_id: int = 0) -> Union[Bu
     Bunch
         Movie reviews and sentiment labels (0 means 'negative' and 1 means 'positive').
     (data, target)
-        Tuple if ``return_X_y`` is true
+        Tuple if ``return_X_y=True``.
     """
     url = MOVIESENTIMENT_URLS[url_id]
     try:
@@ -114,7 +116,7 @@ def fetch_movie_sentiment(return_X_y: bool = False, url_id: int = 0) -> Union[Bu
     labels = []
     for i, member in enumerate(tar.getnames()[1:]):
         f = tar.extractfile(member)
-        for line in f.readlines():
+        for line in f.readlines():  # type: ignore[union-attr]
             try:
                 line.decode('utf8')
             except UnicodeDecodeError:
@@ -129,7 +131,7 @@ def fetch_movie_sentiment(return_X_y: bool = False, url_id: int = 0) -> Union[Bu
     return Bunch(data=data, target=labels, target_names=target_names)
 
 
-def fetch_adult(features_drop: list = None, return_X_y: bool = False, url_id: int = 0) -> \
+def fetch_adult(features_drop: Optional[list] = None, return_X_y: bool = False, url_id: int = 0) -> \
         Union[Bunch, Tuple[np.ndarray, np.ndarray]]:
     """
     Downloads and pre-processes 'adult' dataset.
@@ -138,11 +140,11 @@ def fetch_adult(features_drop: list = None, return_X_y: bool = False, url_id: in
     Parameters
     ----------
     features_drop
-        List of features to be dropped from dataset, by default drops ["fnlwgt", "Education-Num"]
+        List of features to be dropped from dataset, by default drops ``["fnlwgt", "Education-Num"]``.
     return_X_y
-        If true, return features X and labels y as numpy arrays, if False return a Bunch object
+        If ``True``, return features `X` and labels `y` as `numpy` arrays. If ``False`` return a `Bunch` object.
     url_id
-        Index specifying which URL to use for downloading
+        Index specifying which URL to use for downloading.
 
     Returns
     -------
@@ -150,7 +152,7 @@ def fetch_adult(features_drop: list = None, return_X_y: bool = False, url_id: in
         Dataset, labels, a list of features and a dictionary containing a list with the potential categories
         for each categorical feature where the key refers to the feature column.
     (data, target)
-        Tuple if ``return_X_y`` is true
+        Tuple if ``return_X_y=True``
     """
     if features_drop is None:
         features_drop = ["fnlwgt", "Education-Num"]
@@ -249,19 +251,20 @@ def fetch_adult(features_drop: list = None, return_X_y: bool = False, url_id: in
     return Bunch(data=data, target=labels, feature_names=features, target_names=target_names, category_map=category_map)
 
 
-def fetch_fashion_mnist(return_X_y: bool = False):
+def fetch_fashion_mnist(return_X_y: bool = False
+                        ) -> Union[Bunch, Tuple[np.ndarray, np.ndarray]]:
     """
     Loads the Fashion MNIST dataset.
 
     Parameters
     ----------
     return_X_y:
-        If True, an NxMxP array of data points and N-array of labels are returned
+        If ``True``, an `N x M x P` array of data points and `N`-array of labels are returned
         instead of a dict.
 
     Returns
     -------
-    If return_X_y is False, a Bunch object with fields 'data', 'targets' and 'target_names'
+    If ``return_X_y=False``, a Bunch object with fields 'data', 'targets' and 'target_names'
     is returned. Otherwise an array with data points and an array of labels is returned.
     """
 
