@@ -1,11 +1,13 @@
-from alibi.api.interfaces import Explainer, Explanation, FitMixin
-from alibi.api.defaults import DEFAULT_META_CEM, DEFAULT_DATA_CEM
 import copy
 import logging
-import numpy as np
 import sys
+from typing import Any, Callable, Optional, Tuple, Union
+
+import numpy as np
 import tensorflow.compat.v1 as tf
-from typing import Any, Callable, Tuple, Union
+
+from alibi.api.defaults import DEFAULT_DATA_CEM, DEFAULT_META_CEM
+from alibi.api.interfaces import Explainer, Explanation, FitMixin
 
 logger = logging.getLogger(__name__)
 
@@ -20,7 +22,7 @@ class CEM(Explainer, FitMixin):
                  beta: float = .1,
                  feature_range: tuple = (-1e10, 1e10),
                  gamma: float = 0.,
-                 ae_model: Union[tf.keras.Model] = None,
+                 ae_model: Optional[tf.keras.Model] = None,
                  learning_rate_init: float = 1e-2,
                  max_iterations: int = 1000,
                  c_init: float = 10.,
@@ -28,9 +30,9 @@ class CEM(Explainer, FitMixin):
                  eps: tuple = (1e-3, 1e-3),
                  clip: tuple = (-100., 100.),
                  update_num_grad: int = 1,
-                 no_info_val: Union[float, np.ndarray] = None,
-                 write_dir: str = None,
-                 sess: tf.Session = None) -> None:
+                 no_info_val: Optional[Union[float, np.ndarray]] = None,
+                 write_dir: Optional[str] = None,
+                 sess: Optional[tf.Session] = None) -> None:
         """
         Initialize contrastive explanation method.
         Paper: https://arxiv.org/abs/1802.07623
@@ -38,46 +40,46 @@ class CEM(Explainer, FitMixin):
         Parameters
         ----------
         predict
-            Keras or TensorFlow model or any other model's prediction function returning class probabilities
+            `tensorflow` model or any other model's prediction function returning class probabilities.
         mode
-            Find pertinant negatives ('PN') or pertinant positives ('PP')
+            Find pertinent negatives (PN) or pertinent positives (PP).
         shape
-            Shape of input data starting with batch size
+            Shape of input data starting with batch size.
         kappa
-            Confidence parameter for the attack loss term
+            Confidence parameter for the attack loss term.
         beta
-            Regularization constant for L1 loss term
+            Regularization constant for L1 loss term.
         feature_range
-            Tuple with min and max ranges to allow for perturbed instances. Min and max ranges can be floats or
-            numpy arrays with dimension (1x nb of features) for feature-wise ranges
+            Tuple with min and max ranges to allow for perturbed instances. Min and max ranges can be `float` or
+            `numpy` arrays with dimension (1x nb of features) for feature-wise ranges.
         gamma
-            Regularization constant for optional auto-encoder loss term
+            Regularization constant for optional auto-encoder loss term.
         ae_model
-            Optional auto-encoder model used for loss regularization
+            Optional auto-encoder model used for loss regularization.
         learning_rate_init
-            Initial learning rate of optimizer
+            Initial learning rate of optimizer.
         max_iterations
-            Maximum number of iterations for finding a PN or PP
+            Maximum number of iterations for finding a PN or PP.
         c_init
-            Initial value to scale the attack loss term
+            Initial value to scale the attack loss term.
         c_steps
-            Number of iterations to adjust the constant scaling the attack loss term
+            Number of iterations to adjust the constant scaling the attack loss term.
         eps
-            If numerical gradients are used to compute dL/dx = (dL/dp) * (dp/dx), then eps[0] is used to
-            calculate dL/dp and eps[1] is used for dp/dx. eps[0] and eps[1] can be a combination of float values and
-            numpy arrays. For eps[0], the array dimension should be (1x nb of prediction categories) and for
-            eps[1] it should be (1x nb of features)
+            If numerical gradients are used to compute `dL/dx = (dL/dp) * (dp/dx)`, then `eps[0]` is used to
+            calculate `dL/dp` and `eps[1]` is used for `dp/dx`. `eps[0]` and `eps[1]` can be a combination of `float`
+            values and `numpy` arrays. For `eps[0]`, the array dimension should be (1x nb of prediction categories)
+            and for `eps[1]` it should be (1x nb of features).
         clip
-            Tuple with min and max clip ranges for both the numerical gradients and the gradients
-            obtained from the TensorFlow graph
+            Tuple with `min` and `max` clip ranges for both the numerical gradients and the gradients
+            obtained from the `tensorflow` graph.
         update_num_grad
-            If numerical gradients are used, they will be updated every update_num_grad iterations
+            If numerical gradients are used, they will be updated every `update_num_grad` iterations.
         no_info_val
-            Global or feature-wise value considered as containing no information
+            Global or feature-wise value considered as containing no information.
         write_dir
-            Directory to write tensorboard files to
+            Directory to write `tensorboard` files to.
         sess
-            Optional Tensorflow session that will be used if passed instead of creating or inferring one internally
+            Optional `tensorflow` session that will be used if passed instead of creating or inferring one internally.
         """
         super().__init__(meta=copy.deepcopy(DEFAULT_META_CEM))
         # get params for storage in meta
@@ -121,10 +123,10 @@ class CEM(Explainer, FitMixin):
         self.eps = eps
         self.clip = clip
         self.write_dir = write_dir
-        if type(no_info_val) == float:
+        if isinstance(no_info_val, float):
             self.no_info_val = np.ones(shape) * no_info_val
         else:
-            self.no_info_val = no_info_val
+            self.no_info_val = no_info_val  # type: ignore[assignment]
 
         # values regarded as containing no information
         # PNs will deviate away from these values while PPs will gravitate towards them
@@ -301,9 +303,9 @@ class CEM(Explainer, FitMixin):
         Parameters
         ----------
         train_data
-            Representative sample from the training data
+            Representative sample from the training data.
         no_info_type
-            Median or mean value by feature supported
+            Median or mean value by feature supported.
         """
         # TODO: find equal distance area in distribution to different classes as "no info" area
         if self.no_info_val is not None:
@@ -330,9 +332,9 @@ class CEM(Explainer, FitMixin):
         Parameters
         ----------
         pred_proba
-            Prediction probabilities of an instance
+            Prediction probabilities of an instance.
         Y
-            One-hot representation of instance labels
+            One-hot representation of instance labels.
 
         Returns
         -------
@@ -361,11 +363,12 @@ class CEM(Explainer, FitMixin):
         Parameters
         ----------
         X
-            Array to be perturbed
+            Array to be perturbed.
         eps
-            Size of perturbation
+            Size of perturbation.
         proba
-            If True, the net effect of the perturbation needs to be 0 to keep the sum of the probabilities equal to 1
+            If ``True``, the net effect of the perturbation needs to be 0 to keep the sum of the
+            probabilities equal to 1.
 
         Returns
         -------
@@ -389,14 +392,14 @@ class CEM(Explainer, FitMixin):
     def get_gradients(self, X: np.ndarray, Y: np.ndarray) -> np.ndarray:
         """
         Compute numerical gradients of the attack loss term:
-        dL/dx = (dL/dP)*(dP/dx) with L = loss_attack_s; P = predict; x = adv_s
+        `dL/dx = (dL/dP)*(dP/dx)` with `L = loss_attack_s; P = predict; x = adv_s`
 
         Parameters
         ----------
         X
-            Instance around which gradient is evaluated
+            Instance around which gradient is evaluated.
         Y
-            One-hot representation of instance labels
+            One-hot representation of instance labels.
 
         Returns
         -------
@@ -444,18 +447,17 @@ class CEM(Explainer, FitMixin):
     def attack(self, X: np.ndarray, Y: np.ndarray, verbose: bool = False) \
             -> Tuple[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """
-        Find pertinent negative or pertinent positive for instance X using a fast iterative
+        Find pertinent negative or pertinent positive for instance `X` using a fast iterative
         shrinkage-thresholding algorithm (FISTA).
 
         Parameters
         ----------
         X
-            Instance to attack
+            Instance to attack.
         Y
-            Labels for X
+            Labels for `X`.
         verbose
-            Print intermediate results of optimization if True
-
+            Print intermediate results of optimization if ``True``.
 
         Returns
         -------
@@ -477,9 +479,9 @@ class CEM(Explainer, FitMixin):
             Parameters
             ----------
             x
-                Predicted class probabilities or labels
+                Predicted class probabilities or labels.
             y
-                Target or predicted labels
+                Target or predicted labels.
 
             Returns
             -------
@@ -652,23 +654,27 @@ class CEM(Explainer, FitMixin):
             best_attack = X - best_attack
         return best_attack, overall_best_grad
 
-    def explain(self, X: np.ndarray, Y: np.ndarray = None, verbose: bool = False) -> Explanation:
+    def explain(self, X: np.ndarray, Y: Optional[np.ndarray] = None, verbose: bool = False) -> Explanation:
         """
         Explain instance and return PP or PN with metadata.
 
         Parameters
         ----------
         X
-            Instances to attack
+            Instances to attack.
         Y
-            Labels for X
+            Labels for `X`.
         verbose
-            Print intermediate results of optimization if True
+            Print intermediate results of optimization if ``True``.
 
         Returns
         -------
         explanation
             `Explanation` object containing the PP or PN with additional metadata as attributes.
+            See usage at `CEM examples`_ for details.
+
+            .. _CEM examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/CEM.html
         """
         if X.shape[0] != 1:
             logger.warning('Currently only single instance explanations supported (first dim = 1), '
@@ -713,4 +719,12 @@ class CEM(Explainer, FitMixin):
         return explanation
 
     def reset_predictor(self, predictor: Union[Callable, tf.keras.Model]) -> None:
+        """
+        Resets the predictor function/model.
+
+        Parameters
+        ----------
+        predictor
+            New predictor function/model.
+        """
         raise NotImplementedError('Resetting a predictor is currently not supported')
