@@ -1,17 +1,19 @@
 import copy
 import logging
-import numpy as np
 import sys
-import tensorflow.compat.v1 as tf
-from typing import Any, Callable, Dict, Tuple, Union, Sequence
+from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Union
 
+import numpy as np
+import tensorflow.compat.v1 as tf
+
+from alibi.api.defaults import DEFAULT_DATA_CFP, DEFAULT_META_CFP
 from alibi.api.interfaces import Explainer, Explanation, FitMixin
-from alibi.api.defaults import DEFAULT_META_CFP, DEFAULT_DATA_CFP
 from alibi.confidence import TrustScore
 from alibi.utils.discretizer import Discretizer
-from alibi.utils.distance import abdm, mvdm, multidim_scaling
+from alibi.utils.distance import abdm, multidim_scaling, mvdm
 from alibi.utils.gradients import perturb
-from alibi.utils.mapping import ohe_to_ord_shape, ord_to_num, num_to_ord, ohe_to_ord, ord_to_ohe
+from alibi.utils.mapping import (num_to_ord, ohe_to_ord, ohe_to_ord_shape,
+                                 ord_to_num, ord_to_ohe)
 from alibi.utils.tf import argmax_grad, argmin_grad, one_hot_grad, round_grad
 
 logger = logging.getLogger(__name__)
@@ -38,10 +40,10 @@ class CounterfactualProto(Explainer, FitMixin):
                  beta: float = .1,
                  feature_range: tuple = (-1e10, 1e10),
                  gamma: float = 0.,
-                 ae_model: Union[tf.keras.Model] = None,
-                 enc_model: Union[tf.keras.Model] = None,
+                 ae_model: Optional[tf.keras.Model] = None,
+                 enc_model: Optional[tf.keras.Model] = None,
                  theta: float = 0.,
-                 cat_vars: dict = None,
+                 cat_vars: Optional[Dict[int, int]] = None,
                  ohe: bool = False,
                  use_kdtree: bool = False,
                  learning_rate_init: float = 1e-2,
@@ -51,62 +53,61 @@ class CounterfactualProto(Explainer, FitMixin):
                  eps: tuple = (1e-3, 1e-3),
                  clip: tuple = (-1000., 1000.),
                  update_num_grad: int = 1,
-                 write_dir: str = None,
-                 sess: tf.Session = None) -> None:
+                 write_dir: Optional[str] = None,
+                 sess: Optional[tf.Session] = None) -> None:
         """
         Initialize prototypical counterfactual method.
 
         Parameters
         ----------
         predict
-            Keras or TensorFlow model or any other model's prediction function returning class probabilities
+            `tensorflow` model or any other model's prediction function returning class probabilities.
         shape
-            Shape of input data starting with batch size
+            Shape of input data starting with batch size.
         kappa
-            Confidence parameter for the attack loss term
+            Confidence parameter for the attack loss term.
         beta
-            Regularization constant for L1 loss term
+            Regularization constant for L1 loss term.
         feature_range
-            Tuple with min and max ranges to allow for perturbed instances. Min and max ranges can be floats or
-            numpy arrays with dimension (1x nb of features) for feature-wise ranges
+            Tuple with `min` and `max` ranges to allow for perturbed instances. `Min` and `max` ranges can be `float`
+            or `numpy` arrays with dimension (1x nb of features) for feature-wise ranges.
         gamma
-            Regularization constant for optional auto-encoder loss term
+            Regularization constant for optional auto-encoder loss term.
         ae_model
-            Optional auto-encoder model used for loss regularization
+            Optional auto-encoder model used for loss regularization.
         enc_model
-            Optional encoder model used to guide instance perturbations towards a class prototype
+            Optional encoder model used to guide instance perturbations towards a class prototype.
         theta
-            Constant for the prototype search loss term
+            Constant for the prototype search loss term.
         cat_vars
-            Dict with as keys the categorical columns and as values
-            the number of categories per categorical variable.
+            Dict with as keys the categorical columns and as values the number of categories per categorical variable.
         ohe
             Whether the categorical variables are one-hot encoded (OHE) or not. If not OHE, they are
             assumed to have ordinal encodings.
         use_kdtree
-            Whether to use k-d trees for the prototype loss term if no encoder is available
+            Whether to use k-d trees for the prototype loss term if no encoder is available.
         learning_rate_init
-            Initial learning rate of optimizer
+            Initial learning rate of optimizer.
         max_iterations
-            Maximum number of iterations for finding a counterfactual
+            Maximum number of iterations for finding a counterfactual.
         c_init
-            Initial value to scale the attack loss term
+            Initial value to scale the attack loss term.
         c_steps
-            Number of iterations to adjust the constant scaling the attack loss term
+            Number of iterations to adjust the constant scaling the attack loss term.
         eps
-            If numerical gradients are used to compute dL/dx = (dL/dp) * (dp/dx), then eps[0] is used to
-            calculate dL/dp and eps[1] is used for dp/dx. eps[0] and eps[1] can be a combination of float values and
-            numpy arrays. For eps[0], the array dimension should be (1x nb of prediction categories) and for
-            eps[1] it should be (1x nb of features)
+            If numerical gradients are used to compute `dL/dx = (dL/dp) * (dp/dx)`, then `eps[0]` is used to
+            calculate `dL/dp` and `eps[1]` is used for `dp/dx`. `eps[0]` and `eps[1]` can be a combination of `float`
+            values and `numpy` arrays. For `eps[0]`, the array dimension should be (1x nb of prediction categories)
+            and for `eps[1]` it should be (1x nb of features).
         clip
             Tuple with min and max clip ranges for both the numerical gradients and the gradients
-            obtained from the TensorFlow graph
+            obtained from the `tensorflow` graph.
         update_num_grad
-            If numerical gradients are used, they will be updated every update_num_grad iterations
+            If numerical gradients are used, they will be updated every `update_num_grad` iterations.
         write_dir
-            Directory to write tensorboard files to
+            Directory to write `tensorboard` files to.
         sess
-            Optional Tensorflow session that will be used if passed instead of creating or inferring one internally
+            Optional `tensorflow` session that will be used if passed instead of creating or inferring one internally.
         """
         super().__init__(meta=copy.deepcopy(DEFAULT_META_CFP))
         params = locals()
@@ -160,6 +161,7 @@ class CounterfactualProto(Explainer, FitMixin):
             self.is_cat = True
         else:
             self.is_cat = False
+            cat_vars = dict()  # to avoid further None checks
         self.meta['params'].update(is_cat=self.is_cat)
 
         self.shape = shape
@@ -182,16 +184,15 @@ class CounterfactualProto(Explainer, FitMixin):
         self.clip = clip
         self.write_dir = write_dir
 
-        # compute dimensionality after conversion from OHE to ordinal encoding
-        shape = ohe_to_ord_shape(shape, cat_vars=cat_vars, is_ohe=self.ohe)
-
         if self.is_cat:
+            # compute dimensionality after conversion from OHE to ordinal encoding
+            shape = ohe_to_ord_shape(shape, cat_vars=cat_vars, is_ohe=self.ohe)
 
             # define ragged tensor for mapping from categorical to numerical values
-            self.map_cat_to_num = tf.ragged.constant([np.zeros(v) for _, v in self.cat_vars.items()])
+            self.map_cat_to_num = tf.ragged.constant([np.zeros(v) for _, v in cat_vars.items()])
 
             # define placeholder for mapping which can be fed after the fit step
-            max_key = max(cat_vars, key=cat_vars.get)
+            max_key = max(cat_vars)
             self.max_cat = cat_vars[max_key]
             cat_keys = list(cat_vars.keys())
             n_cat = len(cat_keys)
@@ -441,7 +442,7 @@ class CounterfactualProto(Explainer, FitMixin):
                 adv_to_map
                     Instance to map.
                 to_num
-                    Map from categorical to numerical values if True, vice versa if False.
+                    Map from categorical to numerical values if ``True``, vice versa if ``False``.
 
                 Returns
                 -------
@@ -481,7 +482,7 @@ class CounterfactualProto(Explainer, FitMixin):
 
         # variable for target class proto
         if self.enc_model:
-            self.shape_enc = self.enc.predict(np.zeros(self.shape)).shape
+            self.shape_enc = self.enc.predict(np.zeros(self.shape)).shape  # type: ignore[union-attr]
         else:
             self.shape_enc = shape
 
@@ -554,8 +555,8 @@ class CounterfactualProto(Explainer, FitMixin):
             # gamma * AE loss
             if self.ae_model:
                 # run autoencoder
-                self.adv_ae = self.ae(self.adv_cat)
-                self.adv_ae_s = self.ae(self.adv_cat_s)
+                self.adv_ae = self.ae(self.adv_cat)  # type: ignore[misc]
+                self.adv_ae_s = self.ae(self.adv_cat_s)  # type: ignore[misc]
                 if self.is_cat:  # map output autoencoder back to numerical values
                     self.adv_ae = apply_map(self.adv_ae, to_num=True)
                     self.adv_ae_s = apply_map(self.adv_ae_s, to_num=True)
@@ -599,8 +600,10 @@ class CounterfactualProto(Explainer, FitMixin):
 
         with tf.name_scope('loss_prototype') as scope:
             if self.enc_model:
-                self.loss_proto = self.theta * tf.square(tf.norm(self.enc(self.adv_cat) - self.target_proto))
-                self.loss_proto_s = self.theta * tf.square(tf.norm(self.enc(self.adv_cat_s) - self.target_proto))
+                self.loss_proto = self.theta * tf.square(
+                    tf.norm(self.enc(self.adv_cat) - self.target_proto))  # type: ignore[misc]
+                self.loss_proto_s = self.theta * tf.square(
+                    tf.norm(self.enc(self.adv_cat_s) - self.target_proto))  # type: ignore[misc]
             elif self.use_kdtree:
                 self.loss_proto = self.theta * tf.square(tf.norm(self.adv - self.target_proto))
                 self.loss_proto_s = self.theta * tf.square(tf.norm(self.adv_s - self.target_proto))
@@ -654,9 +657,16 @@ class CounterfactualProto(Explainer, FitMixin):
         else:
             self.writer = None
 
-    def fit(self, train_data: np.ndarray, trustscore_kwargs: dict = None, d_type: str = 'abdm',
-            w: float = None, disc_perc: Sequence[Union[int, float]] = (25, 50, 75), standardize_cat_vars: bool = False,
-            smooth: float = 1., center: bool = True, update_feature_range: bool = True) -> "CounterfactualProto":
+    def fit(self,
+            train_data: np.ndarray,
+            trustscore_kwargs: Optional[dict] = None,
+            d_type: str = 'abdm',
+            w: Optional[float] = None,
+            disc_perc: Sequence[Union[int, float]] = (25, 50, 75),
+            standardize_cat_vars: bool = False,
+            smooth: float = 1.,
+            center: bool = True,
+            update_feature_range: bool = True) -> "CounterfactualProto":
         """
         Get prototypes for each class using the encoder or k-d trees.
         The prototypes are used for the encoder loss term or to calculate the optional trust scores.
@@ -668,21 +678,21 @@ class CounterfactualProto(Explainer, FitMixin):
         trustscore_kwargs
             Optional arguments to initialize the trust scores method.
         d_type
-            Pairwise distance metric used for categorical variables. Currently, 'abdm', 'mvdm' and 'abdm-mvdm'
-            are supported. 'abdm' infers context from the other variables while 'mvdm' uses the model predictions.
-            'abdm-mvdm' is a weighted combination of the two metrics.
+            Pairwise distance metric used for categorical variables. Currently, ``'abdm'``, ``'mvdm'`` and
+            ``'abdm-mvdm'`` are supported. ``'abdm'`` infers context from the other variables while ``'mvdm'`` uses
+            the model predictions. ``'abdm-mvdm'`` is a weighted combination of the two metrics.
         w
-            Weight on 'abdm' (between 0. and 1.) distance if d_type equals 'abdm-mvdm'.
+            Weight on ``'abdm'`` (between 0. and 1.) distance if `d_type` equals ``'abdm-mvdm'``.
         disc_perc
-            List with percentiles used in binning of numerical features used for the 'abdm'
-            and 'abdm-mvdm' pairwise distance measures.
+            List with percentiles used in binning of numerical features used for the ``'abdm'``
+            and ``'abdm-mvdm'`` pairwise distance measures.
         standardize_cat_vars
-            Standardize numerical values of categorical variables if True.
+            Standardize numerical values of categorical variables if ``True``.
         smooth
-            Smoothing exponent between 0 and 1 for the distances. Lower values of l will smooth the difference in
+            Smoothing exponent between 0 and 1 for the distances. Lower values will smooth the difference in
             distance metric between different features.
         center
-            Whether to center the scaled distance measures. If False, the min distance for each feature
+            Whether to center the scaled distance measures. If ``False``, the min distance for each feature
             except for the feature with the highest raw max distance will be the lower bound of the
             feature range, but the upper bound will be below the max feature range.
         update_feature_range
@@ -701,7 +711,7 @@ class CounterfactualProto(Explainer, FitMixin):
         else:
             preds = np.argmax(self.predict(train_data), axis=1)
 
-        self.cat_vars_ord = None
+        self.cat_vars_ord = dict()  # type: dict
         if self.is_cat:  # compute distance metrics for categorical variables
 
             if self.ohe:  # convert OHE to ordinal encoding
@@ -734,6 +744,10 @@ class CounterfactualProto(Explainer, FitMixin):
 
             # combined distance measure
             if d_type == 'abdm-mvdm':
+                if w is None:
+                    msg = "Must specify a value for `w` if using d_type='abdm-mvdm'"
+                    raise ValueError(msg)
+
                 # pairwise distances
                 d_abdm = abdm(train_data_bin, self.cat_vars_ord, cat_vars_bin)
                 d_mvdm = mvdm(train_data_ord, preds, self.cat_vars_ord, alpha=1)  # type: ignore
@@ -779,7 +793,7 @@ class CounterfactualProto(Explainer, FitMixin):
             self.d_abs_ragged = np.array(self.d_abs_ragged)
 
         if self.enc_model:
-            enc_data = self.enc.predict(train_data)
+            enc_data = self.enc.predict(train_data)  # type: ignore[union-attr]
             self.class_proto = {}  # type: dict
             self.class_enc = {}  # type: dict
             for i in range(self.classes):
@@ -807,9 +821,9 @@ class CounterfactualProto(Explainer, FitMixin):
         Parameters
         ----------
         pred_proba
-            Prediction probabilities of an instance
+            Prediction probabilities of an instance.
         Y
-            One-hot representation of instance labels
+            One-hot representation of instance labels.
 
         Returns
         -------
@@ -826,17 +840,17 @@ class CounterfactualProto(Explainer, FitMixin):
         return loss_attack
 
     def get_gradients(self, X: np.ndarray, Y: np.ndarray, grads_shape: tuple,
-                      cat_vars_ord: dict = None) -> np.ndarray:
+                      cat_vars_ord: dict) -> np.ndarray:
         """
         Compute numerical gradients of the attack loss term:
-        dL/dx = (dL/dP)*(dP/dx) with L = loss_attack_s; P = predict; x = adv_s
+        `dL/dx = (dL/dP)*(dP/dx)` with `L = loss_attack_s; P = predict; x = adv_s`.
 
         Parameters
         ----------
         X
-            Instance around which gradient is evaluated
+            Instance around which gradient is evaluated.
         Y
-            One-hot representation of instance labels
+            One-hot representation of instance labels.
         grads_shape
             Shape of gradients.
         cat_vars_ord
@@ -854,7 +868,6 @@ class CounterfactualProto(Explainer, FitMixin):
                 X_pred = ord_to_ohe(X_pred, cat_vars_ord)[0]
         else:
             X_pred = X
-
         # N = gradient batch size; F = nb of features; P = nb of prediction classes; B = instance batch size
         # dL/dP -> BxP
         preds = self.predict(X_pred)  # NxP
@@ -903,17 +916,17 @@ class CounterfactualProto(Explainer, FitMixin):
         Parameters
         ----------
         X
-            Instance to encode and calculate distance metrics for
+            Instance to encode and calculate distance metrics for.
         adv_class
-            Predicted class on the perturbed instance
+            Predicted class on the perturbed instance.
         orig_class
-            Predicted class on the original instance
+            Predicted class on the original instance.
         eps
-            Small number to avoid dividing by 0
+            Small number to avoid dividing by 0.
 
         Returns
         -------
-        Ratio between the distance to the prototype of the predicted class for the original instance and
+        Ratio between the distance to the prototype of the predicted class for the original instance and \
         the prototype of the predicted class for the perturbed instance.
         """
         if self.enc_model:
@@ -921,7 +934,7 @@ class CounterfactualProto(Explainer, FitMixin):
                 X = num_to_ord(X, self.d_abs)
             if self.ohe:
                 X, _ = ord_to_ohe(X, self.cat_vars_ord)  # TODO: (Arnaud) is this a genuine bug?
-            X_enc = self.enc.predict(X)
+            X_enc = self.enc.predict(X)  # type: ignore[union-attr]
             adv_proto = self.class_proto[adv_class]
             orig_proto = self.class_proto[orig_class]
             dist_adv = np.linalg.norm(X_enc - adv_proto)
@@ -931,29 +944,29 @@ class CounterfactualProto(Explainer, FitMixin):
             dist_orig = self.kdtrees[orig_class].query(X, k=1)[0]
         else:
             logger.warning('Need either an encoder or the k-d trees enabled to compute distance scores.')
-        return dist_orig / (dist_adv + eps)
+        return dist_orig / (dist_adv + eps)  # type: ignore[return-value]
 
-    def attack(self, X: np.ndarray, Y: np.ndarray, target_class: list = None, k: int = None, k_type: str = 'mean',
-               threshold: float = 0., verbose: bool = False, print_every: int = 100, log_every: int = 100) \
-            -> Tuple[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
+    def attack(self, X: np.ndarray, Y: np.ndarray, target_class: Optional[list] = None, k: Optional[int] = None,
+               k_type: str = 'mean', threshold: float = 0., verbose: bool = False, print_every: int = 100,
+               log_every: int = 100) -> Tuple[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """
-        Find a counterfactual (CF) for instance X using a fast iterative shrinkage-thresholding algorithm (FISTA).
+        Find a counterfactual (CF) for instance `X` using a fast iterative shrinkage-thresholding algorithm (FISTA).
 
         Parameters
         ----------
         X
-            Instance to attack
+            Instance to attack.
         Y
-            Labels for X as one-hot-encoding
+            Labels for `X` as one-hot-encoding.
         target_class
-            List with target classes used to find closest prototype. If None, the nearest prototype
+            List with target classes used to find closest prototype. If ``None``, the nearest prototype
             except for the predict class on the instance is used.
         k
             Number of nearest instances used to define the prototype for a class. Defaults to using all
             instances belonging to the class if an encoder is used and to 1 for k-d trees.
         k_type
-            Use either the average encoding of the k nearest instances in a class (k_type='mean') or
-            the k-nearest encoding in the class (k_type='point') to define the prototype of that class.
+            Use either the average encoding of the k nearest instances in a class (``k_type='mean'``) or
+            the k-nearest encoding in the class (``k_type='point'``) to define the prototype of that class.
             Only relevant if an encoder is used to define the prototypes.
         threshold
             Threshold level for the ratio between the distance of the counterfactual to the prototype of the
@@ -961,11 +974,11 @@ class CounterfactualProto(Explainer, FitMixin):
             for the counterfactual. If the trust score is below the threshold, the proposed counterfactual does
             not meet the requirements.
         verbose
-            Print intermediate results of optimization if True
+            Print intermediate results of optimization if ``True``.
         print_every
-            Print frequency if verbose is True
+            Print frequency if verbose is ``True``.
         log_every
-            Tensorboard log frequency if write directory is specified
+            `tensorboard` log frequency if write directory is specified.
 
         Returns
         -------
@@ -982,9 +995,9 @@ class CounterfactualProto(Explainer, FitMixin):
             Parameters
             ----------
             x
-                Predicted class probabilities or labels
+                Predicted class probabilities or labels.
             y
-                Target or predicted labels
+                Target or predicted labels.
 
             Returns
             -------
@@ -1005,6 +1018,7 @@ class CounterfactualProto(Explainer, FitMixin):
                 print('Target classes: {}'.format(target_class))
 
         if self.is_cat and self.ohe:  # map categorical to numerical data
+
             X_ord = ohe_to_ord(X, self.cat_vars)[0]
             X_num = ord_to_num(X_ord, self.d_abs)
         elif self.is_cat:
@@ -1016,7 +1030,7 @@ class CounterfactualProto(Explainer, FitMixin):
         dist_proto = {}
         if self.enc_model:
 
-            X_enc = self.enc.predict(X)
+            X_enc = self.enc.predict(X)  # type: ignore[union-attr]
             class_dict = self.class_proto if k is None else self.class_enc
 
             for c, v in class_dict.items():
@@ -1045,7 +1059,8 @@ class CounterfactualProto(Explainer, FitMixin):
                 self.class_proto[c] = self.X_by_class[c][idx_c[0][-1]].reshape(1, -1)
 
         if self.enc_or_kdtree:
-            self.id_proto = min(dist_proto, key=dist_proto.get)
+            self.id_proto = min(dist_proto)
+            self.id_proto = min(dist_proto)
             proto_val = self.class_proto[self.id_proto]
             if verbose:
                 print('Prototype class: {}'.format(self.id_proto))
@@ -1256,27 +1271,34 @@ class CounterfactualProto(Explainer, FitMixin):
 
         return best_attack, overall_best_grad
 
-    def explain(self, X: np.ndarray, Y: np.ndarray = None, target_class: list = None, k: int = None,
-                k_type: str = 'mean', threshold: float = 0., verbose: bool = False,
-                print_every: int = 100, log_every: int = 100) -> Explanation:
+    def explain(self,
+                X: np.ndarray,
+                Y: Optional[np.ndarray] = None,
+                target_class: Optional[list] = None,
+                k: Optional[int] = None,
+                k_type: str = 'mean',
+                threshold: float = 0.,
+                verbose: bool = False,
+                print_every: int = 100,
+                log_every: int = 100) -> Explanation:
         """
         Explain instance and return counterfactual with metadata.
 
         Parameters
         ----------
         X
-            Instances to attack
+            Instances to attack.
         Y
-            Labels for X as one-hot-encoding
+            Labels for `X` as one-hot-encoding.
         target_class
-            List with target classes used to find closest prototype. If None, the nearest prototype
+            List with target classes used to find closest prototype. If ``None``, the nearest prototype
             except for the predict class on the instance is used.
         k
             Number of nearest instances used to define the prototype for a class. Defaults to using all
             instances belonging to the class if an encoder is used and to 1 for k-d trees.
         k_type
-            Use either the average encoding of the k nearest instances in a class (k_type='mean') or
-            the k-nearest encoding in the class (k_type='point') to define the prototype of that class.
+            Use either the average encoding of the `k` nearest instances in a class (``k_type='mean'``) or
+            the k-nearest encoding in the class (``k_type='point'``) to define the prototype of that class.
             Only relevant if an encoder is used to define the prototypes.
         threshold
             Threshold level for the ratio between the distance of the counterfactual to the prototype of the
@@ -1284,16 +1306,20 @@ class CounterfactualProto(Explainer, FitMixin):
             for the counterfactual. If the trust score is below the threshold, the proposed counterfactual does
             not meet the requirements.
         verbose
-            Print intermediate results of optimization if True
+            Print intermediate results of optimization if ``True``.
         print_every
-            Print frequency if verbose is True
+            Print frequency if verbose is ``True``.
         log_every
-            Tensorboard log frequency if write directory is specified
+            `tensorboard` log frequency if write directory is specified
 
         Returns
         -------
         explanation
             `Explanation` object containing the counterfactual with additional metadata as attributes.
+            See usage at `CFProto examples`_ for details.
+
+            .. _CFProto examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/CFProto.html
         """
         # get params for storage in meta
         params = locals()
@@ -1356,4 +1382,12 @@ class CounterfactualProto(Explainer, FitMixin):
         return explanation
 
     def reset_predictor(self, predictor: Union[Callable, tf.keras.Model]) -> None:
+        """
+        Resets the predictor function/model.
+
+        Parameters
+        ----------
+        predictor
+            New predictor function/model.
+        """
         raise NotImplementedError('Resetting a predictor is currently not supported')
