@@ -1,20 +1,23 @@
-import os
 import logging
-import numpy as np
-from tqdm import tqdm  # type: ignore
-from copy import deepcopy
-from typing import Union, Any, Callable, Optional, Tuple, Dict, TYPE_CHECKING
+import os
 from abc import ABC, abstractmethod
+from copy import deepcopy
+from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, Tuple, Union, cast
 
-from alibi.api.defaults import DEFAULT_META_CFRL, DEFAULT_DATA_CFRL
+import numpy as np
+from tqdm import tqdm
+
+from alibi.api.defaults import DEFAULT_DATA_CFRL, DEFAULT_META_CFRL
 from alibi.api.interfaces import Explainer, Explanation, FitMixin
-from alibi.utils.frameworks import has_pytorch, has_tensorflow, Framework
-from alibi.explainers.backends.cfrl_base import identity_function, generate_empty_condition,\
-    get_classification_reward, get_hard_distribution
+from alibi.explainers.backends.cfrl_base import (generate_empty_condition,
+                                                 get_classification_reward,
+                                                 get_hard_distribution,
+                                                 identity_function)
+from alibi.utils.frameworks import Framework, has_pytorch, has_tensorflow
 
 if TYPE_CHECKING:
-    import torch
     import tensorflow
+    import torch
 
 if has_pytorch:
     # import pytorch backend
@@ -52,7 +55,7 @@ class NormalActionNoise:
         Parameters
         ----------
         shape
-            Shape of the tensor to be generated
+            Shape of the array to be generated
 
         Returns
         -------
@@ -68,9 +71,15 @@ class ReplayBuffer:
     """
     Circular experience replay buffer for `CounterfactualRL` (DDPG). When the buffer is filled, then the oldest
     experience is replaced by the new one (FIFO). The experience batch size is kept constant and inferred when
-    the first batch of data is stored. Allowing flexible batch size can generate Tensorflow warning due to
+    the first batch of data is stored. Allowing flexible batch size can generate `tensorflow` warning due to
     the `tf.function` retracing, which can lead to a drop in performance.
     """
+    X: np.ndarray  #: Inputs buffer.
+    Y_m: np.ndarray  #: Model's prediction buffer.
+    Y_t: np.ndarray  #: Counterfactual targets buffer.
+    Z: np.ndarray  #: Input embedding buffer.
+    Z_cf_tilde: np.ndarray  #: Noised counterfactual embedding buffer.
+    R_tilde: np.ndarray  #: Noise counterfactual rewards buffer.
 
     def __init__(self, size: int = 1000) -> None:
         """
@@ -80,20 +89,14 @@ class ReplayBuffer:
         ----------
         size
             Dimension of the buffer in batch size. This that the total memory allocated is proportional with the
-            `size` * `batch_size`, where `batch_size` is inferred from the first tensors to be stored.
+            `size x batch_size`, where `batch_size` is inferred from the first array to be stored.
         """
-        self.X: Optional[np.ndarray] = None            # buffer for the inputs
-        self.Y_m: Optional[np.ndarray] = None          # buffer for the model's prediction
-        self.Y_t: Optional[np.ndarray] = None          # buffer for the counterfactual targets
-        self.Z: Optional[np.ndarray] = None            # buffer for the input embedding
-        self.Z_cf_tilde: Optional[np.ndarray] = None   # buffer for the noised counterfactual embedding
-        self.C: Optional[np.ndarray] = None            # buffer for the conditional tensor
-        self.R_tilde: Optional[np.ndarray] = None      # buffer for the noised counterfactual reward tensor
 
-        self.idx = 0                    # cursor for the buffer
-        self.len = 0                    # current length of the buffer
-        self.size = size                # buffer's maximum capacity
-        self.batch_size = 0             # batch size (inferred during `append`)
+        self.idx = 0  # cursor for the buffer
+        self.len = 0  # current length of the buffer
+        self.size = size  # buffer's maximum capacity
+        self.batch_size = 0  # batch size (inferred during `append`)
+        self.C: Optional[np.ndarray] = None  # buffer for the conditional tensor
 
     def append(self,
                X: np.ndarray,
@@ -113,7 +116,7 @@ class ReplayBuffer:
         X
             Input array.
         Y_m
-            Model's prediction class of x.
+            Model's prediction class of `X`.
         Y_t
             Counterfactual target class.
         Z
@@ -124,9 +127,11 @@ class ReplayBuffer:
             Conditional array.
         R_tilde
             Noised counterfactual reward array.
+        **kwargs
+            Other arguments. Not used.
         """
         # Initialize the buffers.
-        if self.X is None:
+        if not hasattr(self, 'X'):
             self.batch_size = X.shape[0]
 
             # Allocate memory.
@@ -157,6 +162,7 @@ class ReplayBuffer:
         self.R_tilde[start:start + self.batch_size] = R_tilde
 
         if C is not None:
+            self.C = cast(np.ndarray, self.C)  # helping mypy out as self.C cannot be None at this point
             self.C[start:start + self.batch_size] = C
 
         # Compute the next index. Not that if the buffer reached its maximum capacity, for the next iteration
@@ -169,21 +175,21 @@ class ReplayBuffer:
 
         Returns
         -------
-            A batch experience. For a description of the keys and values returned, see parameter descriptions \
-            in :py:meth:`alibi.explainers.cfrl_base.ReplayBuffer.append` method. The batch size returned is the same \
-            as the one passed in the :py:meth:`alibi.explainers.cfrl_base.ReplayBuffer.append`.
+        A batch experience. For a description of the keys and values returned, see parameter descriptions \
+        in :py:meth:`alibi.explainers.cfrl_base.ReplayBuffer.append` method. The batch size returned is the same \
+        as the one passed in the :py:meth:`alibi.explainers.cfrl_base.ReplayBuffer.append`.
         """
         # Generate random indices to be sampled.
         rand_idx = np.random.randint(low=0, high=self.len * self.batch_size, size=(self.batch_size,))
 
         # Extract data form buffers.
-        X = self.X[rand_idx]                                        # input array
-        Y_m = self.Y_m[rand_idx]                                    # model's prediction
-        Y_t = self.Y_t[rand_idx]                                    # counterfactual target
-        Z = self.Z[rand_idx]                                        # input embedding
-        Z_cf_tilde = self.Z_cf_tilde[rand_idx]                      # noised counterfactual embedding
-        C = self.C[rand_idx] if (self.C is not None) else None      # conditional array if exists
-        R_tilde = self.R_tilde[rand_idx]                            # noised counterfactual reward
+        X = self.X[rand_idx]                                    # input array
+        Y_m = self.Y_m[rand_idx]                                # model's prediction
+        Y_t = self.Y_t[rand_idx]                                # counterfactual target
+        Z = self.Z[rand_idx]                                    # input embedding
+        Z_cf_tilde = self.Z_cf_tilde[rand_idx]                  # noised counterfactual embedding
+        C = self.C[rand_idx] if (self.C is not None) else None  # conditional array if exists
+        R_tilde = self.R_tilde[rand_idx]                        # noised counterfactual reward
 
         return {
             "X": X,
@@ -227,73 +233,75 @@ DEFAULT_BASE_PARAMS = {
 """
 Default Counterfactual with Reinforcement Learning parameters.
 
-    - ``'act_noise'``: float, standard deviation for the normal noise added to the actor for exploration.
+    - ``'act_noise'`` : ``float`` - standard deviation for the normal noise added to the actor for exploration.
 
-    - ``'act_low'``: float, minimum action value. Each action component takes values between `[act_low, act_high]`.
+    - ``'act_low'`` : ``float`` - minimum action value. Each action component takes values between \
+     `[act_low, act_high]`.
 
-    - ``'act_high'``: float, maximum action value. Each action component takes values between `[act_low, act_high]`.
+    - ``'act_high'`` : ``float`` - maximum action value. Each action component takes values between \
+    `[act_low, act_high]`.
 
-    - ``'replay_buffer_size'``: int, dimension of the replay buffer in `batch_size` units. The total memory \
-    allocated is proportional with the `size` * `batch_size`.
+    - ``'replay_buffer_size'`` : ``int`` - dimension of the replay buffer in `batch_size` units. The total memory \
+    allocated is proportional with the `size x batch_size`.
 
-    - ``'batch_size'``: int, training batch size.
+    - ``'batch_size'`` : ``int`` -  training batch size.
 
-    - ``'num_workers'``: int, number of workers used by the data loader if `pytorch` backend is selected.
+    - ``'num_workers'`` : ``int`` - number of workers used by the data loader if ``'pytorch'`` backend is selected.
 
-    - ``'shuffle'``: bool, whether to shuffle the datasets every epoch.
+    - ``'shuffle'`` : ``bool`` - whether to shuffle the datasets every epoch.
 
-    - ``'exploration_steps'``: int, number of exploration steps. For the firts `exploration_steps`, the \
+    - ``'exploration_steps'`` : ``int`` - number of exploration steps. For the first `exploration_steps`, the \
     counterfactual embedding coordinates are sampled uniformly at random from the interval `[act_low, act_high]`.
 
-    - ``'update_every'``: int, number of steps that should elapse between gradient updates. Regardless of the \
+    - ``'update_every'`` : ``int`` - number of steps that should elapse between gradient updates. Regardless of the \
     waiting steps, the ratio of waiting steps to gradient steps is locked to 1.
 
-    - ``'update_after'``: int, number of steps to wait before start updating the actor and critic. This ensures that \
-    the replay buffers is full enough for useful updates.
+    - ``'update_after'`` : ``int`` - number of steps to wait before start updating the actor and critic. This ensures \
+    that the replay buffers is full enough for useful updates.
 
-    - ``'backend'``: str, backend to be used: `tensorflow` | `pytorch`. Default `tensorflow`.
+    - ``'backend'`` : ``str`` -  backend to be used: ``'tensorflow'`` | ``'pytorch'``. Default ``'tensorflow'``.
 
-    - ``'train_steps'``: int, number of train steps.
+    - ``'train_steps'`` : ``int`` -  number of train steps.
 
-    - ``'encoder_preprocessor'``: Callable, encoder/autoencoder data preprocessors. Transforms the input data into the \
-    format expected by the autoencoder. By default, the identity function.
+    - ``'encoder_preprocessor'`` : ``Callable`` - encoder/auto-encoder data preprocessors. Transforms the input data \
+    into the format expected by the auto-encoder. By default, the identity function.
 
-    - ``'decoder_inv_preprocessor'``: Callable, decoder/autoencoder data inverse preprocessor. Transforms data from \
-    the autoencoder output format to the original input format. Before calling the prediction function, the data is \
-    inverse preprocessed to match the original input format. By default, the identity function.
+    - ``'decoder_inv_preprocessor'`` : ``Callable`` - decoder/auto-encoder data inverse preprocessor. Transforms data \
+    from the auto-encoder output format to the original input format. Before calling the prediction function, the \
+    data is inverse preprocessed to match the original input format. By default, the identity function.
 
-    - ``'reward_func'``: Callable, element-wise reward function. By default, considers classification task and \
+    - ``'reward_func'`` : ``Callable`` - element-wise reward function. By default, considers classification task and \
     checks if the counterfactual prediction label matches the target label. Note that this is element-wise, so a \
     tensor is expected to be returned.
 
-    - ``'postprocessing_funcs'``: List[Postprocessing], list of post-processing functions. The function are applied in \
-    the order, from low to high index. Non-differentiable post-processing can be applied. The function expects as \
-    arguments `X_cf` - the counterfactual instance, `X` - the original input instance and `C` - the conditional \
-    vector, and returns the post-processed counterfactual instance `X_cf_pp` which is passed as `X_cf` for the \
-    following functions. By default, no post-processing is applied (empty list).
+    - ``'postprocessing_funcs'`` : ``List[Postprocessing]`` - list of post-processing functions. The function are \
+    applied in the order, from low to high index. Non-differentiable post-processing can be applied. The function \
+    expects as arguments `X_cf` - the counterfactual instance, `X` - the original input instance and `C` - the \
+    conditional vector, and returns the post-processed counterfactual instance `X_cf_pp` which is passed as `X_cf` \
+    for the following functions. By default, no post-processing is applied (empty list).
 
-    - ``'conditional_func'``: Callable, generates a conditional vector given a pre-processed input instance. By \
-    default, the function returns `None` which is equivalent to no conditioning.
+    - ``'conditional_func'`` : ``Callable`` - generates a conditional vector given a pre-processed input instance. By \
+    default, the function returns ``None`` which is equivalent to no conditioning.
 
-    - ``'callbacks'``: List[Callback], list of callback functions applied at the end of each training step.
+    - ``'callbacks'`` : ``List[Callback]`` - list of callback functions applied at the end of each training step.
 
-    - ``'actor'``: Optional[Union[tensorflow.keras.Model, torch.nn.Module]], actor network.
+    - ``'actor'`` : ``Optional[Union[tensorflow.keras.Model, torch.nn.Module]]`` - actor network.
 
-    - ``'critic;``: Optional[Union[tensorflow.keras.Model, torch.nn.Module]], critic network.
+    - ``'critic;`` : ``Optional[Union[tensorflow.keras.Model, torch.nn.Module]]`` - critic network.
 
-    - ``'optimizer_actor'``: Optional[Union[tensorflow.keras.optimizers.Optimizer, torch.optim.Optimizer]], actor \
-    optimizer.
+    - ``'optimizer_actor'`` : ``Optional[Union[tensorflow.keras.optimizers.Optimizer, torch.optim.Optimizer]]`` - \
+    actor optimizer.
 
-    - ``'optimizer_critic'``: Optional[Union[tensorflow.keras.optimizer.Optimizer, torch.optim.Optimizer]], critic \
-    optimizer.
+    - ``'optimizer_critic'`` : ``Optional[Union[tensorflow.keras.optimizer.Optimizer, torch.optim.Optimizer]]`` - \
+    critic optimizer.
 
-    - ``'lr_actor'``: float, actor learning rate.
+    - ``'lr_actor'`` : ``float`` - actor learning rate.
 
-    - ``'lr_critic'``: float, critic learning rate.
+    - ``'lr_critic'`` : ``float`` - critic learning rate.
 
-    - ``'actor_hidden_dim'``: int, actor hidden layer dimension.
+    - ``'actor_hidden_dim'`` : ``int`` - actor hidden layer dimension.
 
-    - ``'critic_hidden_dim'``: int, critic hidden layer dimension.
+    - ``'critic_hidden_dim'`` : ``int`` - critic hidden layer dimension.
 """
 
 _PARAM_TYPES = {
@@ -311,9 +319,10 @@ _PARAM_TYPES = {
 """
 Parameter types for serialization
 
-    - ``'primitives'``: List[str], list of parameters having primitive data types.
+    - ``'primitives'`` : List[str] - list of parameters having primitive data types.
 
-    - ``'complex'``: List[str], list of parameters having complex data types (e.g., functions, models, optimizers etc.)
+    - ``'complex'`` : List[str] - list of parameters having complex data types (e.g., functions, models,\
+    optimizers etc.).
 """
 
 
@@ -336,10 +345,10 @@ class CounterfactualRL(Explainer, FitMixin):
         Parameters
         ----------
         predictor
-            A callable that takes a tensor of N data points as inputs and returns N outputs. For classification task,
-            the second dimension of the output should match the number of classes. Thus, the output can be either
-            a soft label distribution or a hard label distribution (i.e. one-hot encoding) without affecting the
-            performance since `argmax` is applied to the predictor's output.
+            A callable that takes a `numpy` array of `N` data points as inputs and returns `N` outputs. For
+            classification task, the second dimension of the output should match the number of classes. Thus, the
+            output can be either a soft label distribution or a hard label distribution (i.e. one-hot encoding)
+            without affecting the performance since `argmax` is applied to the predictor's output.
         encoder
             Pretrained encoder network.
         decoder
@@ -349,12 +358,12 @@ class CounterfactualRL(Explainer, FitMixin):
         coeff_consistency
             Consistency loss coefficient.
         latent_dim
-            Autoencoder latent dimension. Can be omitted if the actor network is user specified.
+            Auto-encoder latent dimension. Can be omitted if the actor network is user specified.
         backend
-            Deep learning backend: `tensorflow` | `pytorch`. Default `tensorflow`.
+            Deep learning backend: ``'tensorflow'`` | ``'pytorch'``. Default ``'tensorflow'``.
         seed
-            Seed for reproducibility. The results are not reproducible for `tensorflow` backend.
-        kwargs
+            Seed for reproducibility. The results are not reproducible for ``'tensorflow'`` backend.
+        **kwargs
             Used to replace any default parameter from :py:data:`alibi.explainers.cfrl_base.DEFAULT_BASE_PARAMS`.
         """
         super().__init__(meta=deepcopy(DEFAULT_META_CFRL))
@@ -401,7 +410,7 @@ class CounterfactualRL(Explainer, FitMixin):
     @staticmethod
     def _serialize_params(params: Dict[str, Any]) -> Dict[str, Any]:
         """
-        Parameter serialization. The function replaces object by human-readable representation
+        Parameter serialization. The function replaces object by human-readable representation.
 
         Parameters
         ----------
@@ -436,7 +445,7 @@ class CounterfactualRL(Explainer, FitMixin):
     def _get_name(a: Any) -> str:
         """
         Constructs a name for the given object. If the object has as built-in name, the name is return.
-        If the object has a built-in class name, the name of the class is returned. Otherwise `unknown` is returned.
+        If the object has a built-in class name, the name of the class is returned. Otherwise ``'unknown'`` is returned.
 
         Parameters
         ----------
@@ -456,7 +465,7 @@ class CounterfactualRL(Explainer, FitMixin):
         return "unknown"
 
     @staticmethod
-    def _verify_backend(backend):
+    def _verify_backend(backend: str):
         """
         Verifies if the backend is supported.
 
@@ -476,14 +485,16 @@ class CounterfactualRL(Explainer, FitMixin):
         elif backend not in [Framework.PYTORCH, Framework.TENSORFLOW]:
             raise NotImplementedError(f'{backend} not implemented. Use `tensorflow` or `pytorch` instead.')
 
-    def _select_backend(self, backend, **kwargs):
+    def _select_backend(self, backend: str, **kwargs):
         """
         Selects the backend according to the `backend` flag.
 
         Parameters
         ---------
         backend
-            Deep learning backend: `tensorflow` | `pytorch`. Default `tensorflow`.
+            Deep learning backend: ``'tensorflow'`` | ``'pytorch'``. Default `tensorflow`.
+        **kwargs
+            Other arguments. Not used.
         """
         return tensorflow_base_backend if backend == "tensorflow" else pytorch_base_backend
 
@@ -491,7 +502,7 @@ class CounterfactualRL(Explainer, FitMixin):
                          predictor: Callable,
                          encoder: 'Union[tensorflow.keras.Model, torch.nn.Module]',
                          decoder: 'Union[tensorflow.keras.Model, torch.nn.Module]',
-                         latent_dim: float,
+                         latent_dim: Optional[int],
                          coeff_sparsity: float,
                          coeff_consistency: float,
                          backend: str,
@@ -503,19 +514,21 @@ class CounterfactualRL(Explainer, FitMixin):
         Parameters
         ----------
         predictor.
-            A callable that takes a tensor of N data points as inputs and returns N outputs.
+            A callable that takes a `numpy` array of `N` data points as inputs and returns `N` outputs.
         encoder
             Pretrained encoder network.
         decoder
             Pretrained decoder network.
         latent_dim
-            Autoencoder latent dimension.
+            Auto-encoder latent dimension.
         coeff_sparsity
             Sparsity loss coefficient.
         coeff_consistency
             Consistency loss coefficient.
         backend
-            Deep learning backend: `tensorflow` | `pytorch`.
+            Deep learning backend: ``'tensorflow'`` | ``'pytorch'``.
+        **kwargs
+            Other arguments.
         """
         # Copy default parameters.
         params = deepcopy(DEFAULT_BASE_PARAMS)
@@ -592,12 +605,12 @@ class CounterfactualRL(Explainer, FitMixin):
 
     def reset_predictor(self, predictor: Any) -> None:
         """
-        Resets the predictor to be explained.
+        Resets the predictor.
 
         Parameters
         ----------
         predictor
-            New predictor to be set.
+            New predictor.
         """
         self.params["predictor"] = predictor
         self.meta["params"].update(CounterfactualRL._serialize_params(self.params))
@@ -750,7 +763,7 @@ class CounterfactualRL(Explainer, FitMixin):
             raise ValueError(f"Target shape should be at least 1 and at most 2. Found {len(Y_t.shape)} instead.")
 
     @staticmethod
-    def _validate_condition(C: np.ndarray):
+    def _validate_condition(C: Optional[np.ndarray]):
         """
         Validate condition vector.
 
@@ -774,14 +787,14 @@ class CounterfactualRL(Explainer, FitMixin):
 
         Returns
         -------
-        `True` if the prediction has shape of 2 and the second dimension bigger grater than 1. `False` otherwise.
+        ``True`` if the prediction has shape of 2 and the second dimension bigger grater than 1. ``False`` otherwise.
         """
         return len(pred.shape) == 2 and pred.shape[1] > 1
 
-    def explain(self,
+    def explain(self,  # type: ignore[override]
                 X: np.ndarray,
-                Y_t: np.ndarray = None,   # TODO: remove default value (mypy error. explanation in the validation step)
-                C: Optional[Any] = None,  # TODO: narrow down the type from `Any` (explanation in the validation step)
+                Y_t: np.ndarray,
+                C: Optional[np.ndarray] = None,
                 batch_size: int = 100) -> Explanation:
         """
         Explains an input instance
@@ -793,24 +806,21 @@ class CounterfactualRL(Explainer, FitMixin):
         Y_t
             Counterfactual targets.
         C
-            Conditional vectors. If `None`, it means that no conditioning was used during training (i.e. the
-            `conditional_func` returns `None`).
+            Conditional vectors. If ``None``, it means that no conditioning was used during training (i.e. the
+            `conditional_func` returns ``None``).
         batch_size
             Batch size to be used when generating counterfactuals.
 
         Returns
         -------
-        `Explanation` object containing the inputs with the corresponding labels, the counterfactuals with the \
-        corresponding labels, targets and additional metadata.
+        explanation
+            `Explanation` object containing the counterfactual with additional metadata as attributes. \
+            See usage at `CFRL examples`_ for details.
+
+            .. _CFRL examples:
+                https://docs.seldon.io/projects/alibi/en/latest/methods/CFRL.html
         """
         # General validation.
-        #
-        # If `Y_t` doesn't have the default value `None` I will get a warning saying that "Signature of method does not
-        # match the signature of class" (Liskov substitution principle). That's why `Y_t` can be None but None is not a
-        # valid value since the target must be specified.
-        #
-        # `C` can be in fact only `np.ndarray`, but for the tabular case a `Dict[str, List]` is expected.
-        # Similar behavior as in the previous comment.
         self._validate_target(Y_t)
         self._validate_condition(C)
 
@@ -835,7 +845,7 @@ class CounterfactualRL(Explainer, FitMixin):
 
         # Perform prediction in mini-batches.
         n_minibatch = int(np.ceil(X.shape[0] / batch_size))
-        all_results: Dict[str, np.ndarray] = {}
+        all_results: Dict[str, Optional[np.ndarray]] = {}
 
         for i in tqdm(range(n_minibatch)):
             istart, istop = i * batch_size, min((i + 1) * batch_size, X.shape[0])
@@ -853,14 +863,15 @@ class CounterfactualRL(Explainer, FitMixin):
                 if all_results[key] is not None:
                     all_results[key] = np.concatenate([all_results[key], results[key]], axis=0)
 
-        return self._build_explanation(**all_results)
+        # see https://github.com/python/mypy/issues/5382 for the type ignore
+        return self._build_explanation(**all_results)  # type: ignore[arg-type]
 
     def _compute_counterfactual(self,
                                 X: np.ndarray,
                                 Y_t: np.ndarray,
-                                C: Optional[np.ndarray] = None) -> Dict[str, Optional[np.ndarray]]:
+                                C: Optional[np.ndarray] = None) -> Dict[str, Optional[np.ndarray]]:  # TODO: TypedDict
         """
-        Compute counterfactual instance for a given input, target and condition vector
+        Compute counterfactual instance for a given input, target and condition vector.
 
         Parameters
         ----------
@@ -869,14 +880,14 @@ class CounterfactualRL(Explainer, FitMixin):
         Y_t
             Counterfactual targets.
         C
-            Conditional vector. If `None`, it means that no conditioning was used during training (i.e. the
-            `conditional_func` returns `None`).
+            Conditional vector. If ``None``, it means that no conditioning was used during training (i.e. the
+            `conditional_func` returns ``None``).
 
         Returns
         -------
-            Dictionary containing the input instances in the original format, input classification labels,
-            counterfactual instances in the original format, counterfactual classification labels, target labels,
-            conditional vectors.
+        Dictionary containing the input instances in the original format, input classification labels,
+        counterfactual instances in the original format, counterfactual classification labels, target labels,
+        conditional vectors.
         """
         # Save original input for later usage.
         X_orig = X
@@ -937,12 +948,12 @@ class CounterfactualRL(Explainer, FitMixin):
             Y_m_cf = np.argmax(Y_m_cf, axis=1)
 
         return {
-            "X": X_orig,         # input instances
-            "Y_m": Y_m,          # input classification labels
-            "X_cf": X_cf,        # counterfactual instances
-            "Y_m_cf": Y_m_cf,    # counterfactual classification labels
-            "Y_t": Y_t,          # target labels
-            "C": C               # conditional vectors
+            "X": X_orig,        # input instances
+            "Y_m": Y_m,         # input classification labels
+            "X_cf": X_cf,       # counterfactual instances
+            "Y_m_cf": Y_m_cf,   # counterfactual classification labels
+            "Y_t": Y_t,         # target labels
+            "C": C              # conditional vectors
         }
 
     def _build_explanation(self,
@@ -951,7 +962,7 @@ class CounterfactualRL(Explainer, FitMixin):
                            X_cf: np.ndarray,
                            Y_m_cf: np.ndarray,
                            Y_t: np.ndarray,
-                           C: Any) -> Explanation:
+                           C: Optional[np.ndarray]) -> Explanation:
         """
         Builds the explanation of the current object.
 
@@ -968,13 +979,13 @@ class CounterfactualRL(Explainer, FitMixin):
         Y_t
             Target labels.
         C
-            Condition vector. If `None`, it means that no conditioning was used during training (i.e. the
-            `conditional_func` returns `None`).
+            Condition vector. If ``None``, it means that no conditioning was used during training (i.e. the
+            `conditional_func` returns ``None``).
 
         Returns
         -------
-            `Explanation` object containing the inputs with the corresponding labels, the counterfactuals with the
-            corresponding labels, targets and additional metadata.
+        `Explanation` object containing the inputs with the corresponding labels, the counterfactuals with the \
+        corresponding labels, targets and additional metadata.
         """
         data = deepcopy(DEFAULT_DATA_CFRL)
 
@@ -1002,20 +1013,20 @@ class Postprocessing(ABC):
         ----------
         X_cf
            Counterfactual instance. The datatype depends on the output of the decoder. For example, for an image
-           dataset, the output is `np.ndarray`. For a tabular dataset, the output is `List[np.ndarray]` where each
+           dataset, the output is ``np.ndarray``. For a tabular dataset, the output is ``List[np.ndarray]`` where each
            element of the list corresponds to a feature. This corresponds to the decoder's output from the
            heterogeneous autoencoder (see :py:class:`alibi.models.tensorflow.autoencoder.HeAE` and
            :py:class:`alibi.models.pytorch.autoencoder.HeAE`).
         X
            Input instance.
         C
-           Conditional vector. If `None`, it means that no conditioning was used during training (i.e. the
-           `conditional_func` returns `None`).
+           Conditional vector. If ``None``, it means that no conditioning was used during training (i.e. the
+           `conditional_func` returns ``None``).
 
         Returns
         -------
         X_cf
-            Post-processed X_cf.
+            Post-processed `X_cf`.
         """
         pass
 
@@ -1042,47 +1053,47 @@ class Callback(ABC):
             bound to 1.
         model
             CounterfactualRL explainer. All the parameters defined in
-            :py:data:`alibi.explainers.cfrl_base.DEFAULT_BASE_PARAMS` can be accessed through 'model.params'.
+            :py:data:`alibi.explainers.cfrl_base.DEFAULT_BASE_PARAMS` can be accessed through `model.params`.
         sample
             Dictionary of samples used for an update which contains
 
-                - ``'X'``: input instances.
+                - ``'X'`` : ``np.ndarray`` - input instances.
 
-                - ``'Y_m'``: predictor outputs for the input instances.
+                - ``'Y_m'`` : ``np.ndarray`` - predictor outputs for the input instances.
 
-                - ``'Y_t'``: target outputs.
+                - ``'Y_t'`` : ``np.ndarray`` - target outputs.
 
-                - ``'Z'``: input embeddings.
+                - ``'Z'`` : ``np.ndarray`` - input embeddings.
 
-                - ``'Z_cf_tilde'``: noised counterfactual embeddings.
+                - ``'Z_cf_tilde'`` : ``np.ndarray`` - noised counterfactual embeddings.
 
-                - ``'X_cf_tilde'``: noised counterfactual instances obtained ofter decoding the noised counterfactual \
-                embeddings `Z_cf_tilde` and apply post-processing functions.
+                - ``'X_cf_tilde'`` : ``np.ndarray`` - noised counterfactual instances obtained ofter decoding the \
+                noised counterfactual embeddings `Z_cf_tilde` and apply post-processing functions.
 
-                - ``'C'``: conditional vector.
+                - ``'C'`` : ``Optional[np.ndarray]`` - conditional vector.
 
-                - ``'R_tilde'``: reward obtained for the noised counterfactual instances.
+                - ``'R_tilde'`` : ``np.ndarray`` - reward obtained for the noised counterfactual instances.
 
-                - ``'Z_cf'``: counterfactual embeddings.
+                - ``'Z_cf'`` : ``np.ndarray`` - counterfactual embeddings.
 
-                - ``'X_cf'``: counterfactual instances obtained after decoding the countefactual embeddings `Z_cf` and \
-                apply post-processing functions.
+                - ``'X_cf'`` : ``np.ndarray`` - counterfactual instances obtained after decoding the counterfactual \
+                embeddings `Z_cf` and apply post-processing functions.
         losses
             Dictionary of losses which contains
 
-                - ``'loss_actor'``: actor network loss.
+                - ``'loss_actor'`` : ``Callable`` - actor network loss.
 
-                - ``'loss_critic'``: critic network loss.
+                - ``'loss_critic'`` : ``Callable`` - critic network loss.
 
-                - ``'sparsity_loss'``: sparsity loss for the \
+                - ``'sparsity_loss'`` : ``Callable`` - sparsity loss for the \
                 :py:class:`alibi.explainers.cfrl_base.CounterfactualRL` class.
 
-                - ``'sparsity_num_loss'``: numerical features sparsity loss for the \
+                - ``'sparsity_num_loss'`` : ``Callable`` - numerical features sparsity loss for the \
                 :py:class:`alibi.explainers.cfrl_tabular.CounterfactualRLTabular` class.
 
-                - ``'sparsity_cat_loss'``: categorical features sparsity loss for the \
+                - ``'sparsity_cat_loss'`` : ``Callable`` - categorical features sparsity loss for the \
                 :py:class:`alibi.explainers.cfrl_tabular.CounterfactualRLTabular` class.
 
-                - ``'consistency_loss'``: consistency loss if used.
+                - ``'consistency_loss'`` : ``Callable`` - consistency loss if used.
         """
         pass
