@@ -92,7 +92,7 @@ def test_explainer(n_explainer_runs, at_defaults, rf_classifier, explainer, test
 
 
 @pytest.mark.skip(reason='Not testing as performance of distributed anchors (within an instance) not clear.'
-                         'Also, these tests fail intermittently with precision treshold not achieved.')
+                         'Also, these tests fail intermittently with precision threshold not achieved.')
 @pytest.mark.parametrize('ncpu', [2], ids='ncpu={}'.format)
 @pytest.mark.parametrize('predict_type', ('proba', 'class'), ids='predict_type={}'.format)
 @pytest.mark.parametrize('at_defaults', [0.9], ids='threshold={}'.format, indirect=True)
@@ -109,6 +109,8 @@ def test_distributed_anchor_tabular(ncpu,
                                     rf_classifier,
                                     test_instance_idx,
                                     ):
+    # TODO - if we add this test is back in, this conditional should be added as a @pytest.mark.skip
+    # i.e. see test_kernel_distributed_execution in test_shap_wrappers.py.
     if RAY_INSTALLED:
         import ray
 
@@ -218,6 +220,13 @@ def test_sampler(test_instance_idx, anchors, nb_samples, dataset, rf_classifier,
     test_dataset = dataset
     test_dataset_name = test_dataset['metadata']['name']
 
+    # the `explain` call does some more sampler setup before using it so we need to repeat it here
+    sampler = explainer.samplers[0]
+    sampler.set_instance_label(X_test[test_instance_idx])
+    # TODO: the test passes now, but we're not setting other things passed to `explain` like `n_covered_ex`
+    #  or calling explainer._build_sampling_lookups. This suggests either the coupling between `sampler` and
+    #  `explainer` is too strong or this test should be re-written in a different way.
+
     # test sampler setup is correct
     assert len(explainer.samplers) == 1
     sampler = explainer.samplers[0]
