@@ -3,9 +3,9 @@ This module defines a wrapper for transformer-based masked language models used
 strategy. The `LanguageModel` base class defines basic functionalities as loading, storing, and predicting.
 
 Language model's tokenizers usually work at a subword level, and thus, a word can be split into subwords. For example,
-a word can be decomposed as: word = [head_token tail_token_1 tail_token_2 ... tail_token_k]. For language models
-such as DistilbertBaseUncased and BertBaseUncased, the tail tokens can be identified by a special prefix `##`.
-On the other hand, for RobertaBase only the head is prefixed with the special character `Ġ`, thus the tail tokens
+a word can be decomposed as: ``word = [head_token tail_token_1 tail_token_2 ... tail_token_k]``. For language models
+such as `DistilbertBaseUncased` and `BertBaseUncased`, the tail tokens can be identified by a special prefix ``'##'``.
+On the other hand, for `RobertaBase` only the head is prefixed with the special character ``'Ġ'``, thus the tail tokens
 can be identified by the absence of the special token. In this module, we refer to a tail token as a subword prefix.
 We will use the notion of a subword to refer to either a `head` or a `tail` token.
 
@@ -23,7 +23,7 @@ interpretable, we ensure that the head will not end with a subword, and will con
 import abc
 import numpy as np
 from pathlib import Path
-from typing import List, Optional, Tuple, Union
+from typing import Any, Callable, List, Optional, Tuple, Union
 
 import tensorflow as tf
 import transformers
@@ -31,7 +31,15 @@ from transformers import TFAutoModelForMaskedLM, AutoTokenizer
 
 
 class LanguageModel(abc.ABC):
-    SUBWORD_PREFIX = ''
+    SUBWORD_PREFIX = ''  #: Language model subword prefix.
+
+    # We don't type transformers objects here as it would likely require digging into
+    # some private base classes which may change in the future and cause breaking changes.
+    model: Any
+    tokenizer: Any
+    # TODO: from TF 2.6 this has type `tf.types.experimental.GenericFunction`,
+    #  unsure if we can be more specific right now
+    caller: Callable
 
     def __init__(self, model_path: str, preloading: bool = True):
         """
@@ -42,16 +50,17 @@ class LanguageModel(abc.ABC):
         model_path
             `transformers` package model path.
         preloading
-            Whether to preload the online version of the transformer.
-            If `False`, a call to `from_disk` method is expected.
+            Whether to preload the online version of the transformer. If ``False``, a call to `from_disk`
+            method is expected.
         """
         self.model_path = model_path
-        self.model, self.caller, self.tokenizer = None, None, None
 
         if preloading:
             # set model (for performance reasons the `call` method is wrapped in tf.function)
             self.model = TFAutoModelForMaskedLM.from_pretrained(model_path)
-            self.caller = tf.function(self.model.call, experimental_relax_shapes=True)
+
+            # To understand the type: ignore see https://github.com/python/mypy/issues/2427
+            self.caller = tf.function(self.model.call, experimental_relax_shapes=True)  # type: ignore[assignment]
 
             # set tokenizer
             self.tokenizer = AutoTokenizer.from_pretrained(model_path)
@@ -67,7 +76,9 @@ class LanguageModel(abc.ABC):
         """
         # set model (for performance reasons the `call` method is wrapped in tf.function)
         self.model = TFAutoModelForMaskedLM.from_pretrained(path, local_files_only=True)
-        self.caller = tf.function(self.model.call, experimental_relax_shapes=True)
+
+        # To understand the type: ignore see https://github.com/python/mypy/issues/2427
+        self.caller = tf.function(self.model.call, experimental_relax_shapes=True)  # type: ignore[assignment]
 
         # set tokenizer
         self.tokenizer = AutoTokenizer.from_pretrained(path, local_files_only=True)
@@ -93,12 +104,12 @@ class LanguageModel(abc.ABC):
     def is_subword_prefix(self, token: str) -> bool:
         """
         Checks if the given token is a part of the tail of a word. Note that a word can
-        be split in multiple tokens (e.g., word = [head_token tail_token_1 tail_token_2 ... tail_token_k]).
+        be split in multiple tokens (e.g., ``word = [head_token tail_token_1 tail_token_2 ... tail_token_k]``).
         Each language model has a convention on how to mark a tail token. For example
-        DistilbertBaseUncased and BertBaseUncased have the tail tokens prefixed with the special
-        set of characters `##`. On the other hand, for RobertaBase only the head token is prefixed
-        with the special character 'Ġ' and thus we need to check the absence of the prefix to identify
-        the tail tokens. We call those special characters SUBWORD_PREFIX. Due to different conventions,
+        `DistilbertBaseUncased` and `BertBaseUncased` have the tail tokens prefixed with the special
+        set of characters ``'##'``. On the other hand, for `RobertaBase` only the head token is prefixed
+        with the special character ``'Ġ'`` and thus we need to check the absence of the prefix to identify
+        the tail tokens. We call those special characters `SUBWORD_PREFIX`. Due to different conventions,
         this method has to be implemented for each language model. See module docstring for namings.
 
         Parameters
@@ -108,7 +119,7 @@ class LanguageModel(abc.ABC):
 
         Returns
         -------
-        True if the given token is a subword prefix. False otherwise.
+        ``True`` if the given token is a subword prefix. ``False`` otherwise.
         """
         pass
 
@@ -118,9 +129,10 @@ class LanguageModel(abc.ABC):
                     punctuation: str) -> str:
         """
         Given a tokenized text and the starting index of a word, the function selects the entire word.
-        Note that a word is composed of multiple tokens (e.g., word = [head_token tail_token_1
-        tail_token_2 ... tail_token_k]. The tail tokens can be identified based on the
-        presence/absence of SUBWORD_PREFIX. See `is_subword_prefix` for more details.
+        Note that a word is composed of multiple tokens (e.g., ``word = [head_token tail_token_1
+        tail_token_2 ... tail_token_k]``). The tail tokens can be identified based on the
+        presence/absence of `SUBWORD_PREFIX`. See :py:meth:`alibi.utils.lang_model.LanguageModel.is_subword_prefix`
+        for more details.
 
         Parameters
         ----------
@@ -134,7 +146,7 @@ class LanguageModel(abc.ABC):
 
         Returns
         -------
-        The word obtained by concatenation [head_token tail_token_1 tail_token_2 ... tail_token_k].
+        The word obtained by concatenation ``[head_token tail_token_1 tail_token_2 ... tail_token_k]``.
         """
         # define the ending index
         end_idx = start_idx + 1
@@ -169,11 +181,11 @@ class LanguageModel(abc.ABC):
         stopwords:
             List of stop words. The words in this list should be lowercase.
         punctuation
-            Punctuation to be considered. See `select_entire_word`.
+            Punctuation to be considered. See :py:meth:`alibi.utils.lang_model.LanguageModel.select_entire_word`.
 
         Returns
         -------
-        True if the `token` is in the `stopwords` list. False otherwise.
+        ``True`` if the `token` is in the `stopwords` list. ``False`` otherwise.
         """
         if not stopwords:
             return False
@@ -197,7 +209,7 @@ class LanguageModel(abc.ABC):
 
         Returns
         -------
-        True if the `token` is a punctuation. False otherwise.
+        ``True`` if the `token` is a punctuation. ``False`` otherwise.
         """
         token = token.replace(self.SUBWORD_PREFIX, '').strip()
         return all([c in punctuation for c in token])
@@ -278,7 +290,7 @@ class LanguageModel(abc.ABC):
                          vocab_size: int,
                          batch_size: int) -> np.ndarray:
         """
-        Tensorflow language model batch predictions for AnchorText.
+        `Tensorflow` language model batch predictions for `AnchorText`.
 
         Parameters
         ----------
@@ -320,12 +332,12 @@ class DistilbertBaseUncased(LanguageModel):
 
     def __init__(self, preloading: bool = True):
         """
-        Initialize DistilbertBaseUncased.
+        Initialize `DistilbertBaseUncased`.
 
         Parameters
         ----------
         preloading
-            See `LanguageModel` constructor.
+            See :py:meth:`alibi.utils.lang_model.LanguageModel.__init__`.
         """
         super().__init__("distilbert-base-uncased", preloading)
 
@@ -342,12 +354,12 @@ class BertBaseUncased(LanguageModel):
 
     def __init__(self, preloading: bool = True):
         """
-        Initialize BertBaseUncased.
+        Initialize `BertBaseUncased`.
 
         Parameters
         ----------
         preloading
-            See `LanguageModel` constructor.
+            See :py:meth:`alibi.utils.lang_model.LanguageModel.__init__`.
         """
         super().__init__("bert-base-uncased", preloading)
 
@@ -364,12 +376,12 @@ class RobertaBase(LanguageModel):
 
     def __init__(self, preloading: bool = True):
         """
-        Initialize RobertaBase
+        Initialize `RobertaBase`.
 
         Parameters
         ----------
         preloading
-            See `LanguageModel` constructor.
+            See :py:meth:`alibi.utils.lang_model.LanguageModel.__init__` constructor.
         """
         super().__init__("roberta-base", preloading)
 
